#include "Enemy.h"

const int ENEMY_SIGHT_RANGE_ = 10;

#define MAPA(x,y) mapArray[x*SCREEN_TILES_Y + y]
#define DIR_LEFT -1
#define DIR_RIGHT 1
#define DIR_UP -1
#define DIR_DOWN 1
#define DIR_STAND 0
#define PATH_FORWARDS 1
#define PATH_BACKWARDS -1

void initEnemy(Enemy* enemy, EnemyType type, u16 x, u8 y, u8 width, u8 height)
{
    enemy->object.width = width;
    enemy->object.height = height;
    enemy->object.position.x = x;
    enemy->object.position.y = y;
    enemy->previousPosition.x = x;
    enemy->previousPosition.y = y;
    enemy->type = type;
    enemy->currentStep = 0;

    //inital velocity is 0    
    enemy->object.velocity.x = 0;
    enemy->object.velocity.y = 0;
    
    //initial direction is always forwards
    enemy->pathTravelDirection = PATH_FORWARDS;
	enemy->destroyed = FALSE;
}

void drawEnemy(Enemy* enemy)
{
    u8* enemyVideoMemory;
    u16 scrolledXPosition = enemy->object.position.x - (tileScrollX*4);
    
    //make sure that the enemy is on screen before drawing it
    if(scrolledXPosition < SCREEN_X)
    {
        //clearing trail from moving enemy
        if(enemy->type != TURRET)
        {
            if(enemy->previousPosition.x != enemy->object.position.x || enemy->previousPosition.y != enemy->object.position.y)
            {
                enemyVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, enemy->previousPosition.x, enemy->previousPosition.y);
                cpct_drawSolidBox(enemyVideoMemory, backgroundColour, enemy->object.width, enemy->object.height);
            }
        }

        //draw the specified enemy in the enemy graphic set
        enemyVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, enemy->object.position.x, enemy->object.position.y);
        cpct_drawSprite(enemyGraphicSet[enemy->spriteNumber], enemyVideoMemory, enemy->object.width, enemy->object.height);                   
    }
}

void logicEnemy(Enemy* enemy, Object* shipObject)
{    
    u8 enemyInRange = FALSE;
    u8 tileScrollAdjustment = tileScrollX*2;

    //check that the timer has ticked down so that the enemy can start moving
    if(enemy->enemySpawnTime != 0)
    {
        if(enemySpawnTimer >= enemy->enemySpawnTime)
        {
            enemy->enemySpawnTime = 0;
        }
    }
    else
    {
        //turrets dont need to move
        if(enemy->type != TURRET)
        {        
            u8 currentXTile = 0;
            u8 currentYTile = 0;
            u8 nextXTile = 0;
            u8 nextYTile = 0; 

            //update previous position
            enemy->previousPosition.x = enemy->object.position.x;
            enemy->previousPosition.y = enemy->object.position.y;

            //calculate new position based on path       
            currentXTile = enemy->path[enemy->currentStep];
            currentYTile = enemy->path[enemy->currentStep+1];               

            //steps-1 * 2 is a bit odd but it does mean that we don't start going into the next enemies path
            if(enemy->pathTravelDirection == PATH_FORWARDS && enemy->currentStep + 1 < (enemy->pathSteps-1)*2)
            {
                u8 endOfX = FALSE;
                u8 endOfY = FALSE;
                i8 xDirection = 0;
                i8 yDirection = 0;
                nextXTile = enemy->path[enemy->currentStep+2];
                nextYTile = enemy->path[enemy->currentStep+3];

                if(nextXTile > currentXTile)
                {
                    enemy->object.velocity.x = 1;
                    xDirection = DIR_RIGHT;
                    
                }
                else if(nextXTile < currentXTile)
                {
                    enemy->object.velocity.x = -1;
                    xDirection = DIR_LEFT;                
                }
                else
                {
                    enemy->object.velocity.x = 0;
                    xDirection = DIR_STAND;
                }

                if(nextYTile < currentYTile)
                {
                    enemy->object.velocity.y = -1;
                    yDirection = DIR_UP;
                }
                else if(nextYTile > currentYTile)
                {
                    enemy->object.velocity.y = 1;
                    yDirection = DIR_DOWN;
                }
                else
                {
                    enemy->object.velocity.y = 0;
                    yDirection = DIR_STAND;     
                }

                //calculate new x position from x velocity
                if(enemy->object.velocity.x != 0)
                {   
                    enemy->object.position.x += enemy->object.velocity.x;          
                }

                //calculate new y position from y velocity
                if(enemy->object.velocity.y != 0)
                {
                    enemy->object.position.y += enemy->object.velocity.y;
                }


                
                if(xDirection == DIR_LEFT)
                {                
                    if(enemy->object.position.x < currentXTile*TILE_WIDTH_BYTES)
                    {                           
                        enemy->object.velocity.x = 0;
                        endOfX = TRUE;
                    }
                }
                else if(xDirection == DIR_RIGHT)
                { 
                    //this might need to be + TILE_WIDTH_BYTES
                    if(enemy->object.position.x > currentXTile*TILE_WIDTH_BYTES)
                    {                
                        enemy->object.velocity.x = 0;
                        endOfX = TRUE;
                    }
                } 
                else if(xDirection == DIR_STAND)
                {
                    enemy->object.velocity.x = 0;
                    endOfX = TRUE;
                }
                
                if(yDirection == DIR_UP)
                {
                    if(enemy->object.position.y < currentYTile*TILE_HEIGHT)
                    {
                        enemy->object.velocity.y = 0;
                        endOfY = TRUE;
                    }
                }
                else if(yDirection == DIR_DOWN)
                {
                    //this might need to be + TILE_HEIGHT
                    if(enemy->object.position.y > currentYTile*TILE_HEIGHT)
                    {
                        enemy->object.velocity.y = 0;
                        endOfY = TRUE;
                    }
                }
                else if(yDirection == DIR_STAND)
                {
                    enemy->object.velocity.y = 0;
                    endOfY = TRUE;
                }

                if(endOfX && endOfY)
                {
                    endOfX = FALSE;
                    endOfY = FALSE;
                    enemy->currentStep += 2;  
                }
            }
            else
            {            
                //todo reached the end of the path
                if(enemy->type == FLYING_AND_OFF)
                {
                    //TODO destroy the enemy as it is at the end of its path
                }
                else if(enemy->type == FLYING_REPEAT)
                {
                    //return enemy to the start of the path
                    enemy->pathTravelDirection = PATH_BACKWARDS;
                }
            }

            //this is used for enemies that travel forward and backwards instead of disappearing off the map
            if(enemy->pathTravelDirection == PATH_BACKWARDS && enemy->currentStep - 1 > 0)
            {
                u8 endOfX = FALSE;
                u8 endOfY = FALSE;
                i8 xDirection = 0;
                i8 yDirection = 0;
                nextXTile = enemy->path[enemy->currentStep-2];
                nextYTile = enemy->path[enemy->currentStep-1];

                if(nextXTile > currentXTile)
                {
                    enemy->object.velocity.x = 1;
                    xDirection = DIR_RIGHT;                
                }
                else if(nextXTile < currentXTile)
                {
                    enemy->object.velocity.x = -1;
                    xDirection = DIR_LEFT;                
                }
                else
                {
                    enemy->object.velocity.x = 0;
                    xDirection = DIR_STAND;
                }

                if(nextYTile < currentYTile)
                {
                    enemy->object.velocity.y = -1;
                    yDirection = DIR_UP;
                }
                else if(nextYTile > currentYTile)
                {
                    enemy->object.velocity.y = 1;
                    yDirection = DIR_DOWN;
                }
                else
                {
                    enemy->object.velocity.y = 0;
                    yDirection = DIR_STAND;     
                }

                //calculate new x position from x velocity
                if(enemy->object.velocity.x != 0)
                {   
                    enemy->object.position.x += enemy->object.velocity.x;          
                }

                //calculate new y position from y velocity
                if(enemy->object.velocity.y != 0)
                {
                    enemy->object.position.y += enemy->object.velocity.y;
                }

                
                if(xDirection == DIR_LEFT)
                {                
                    if(enemy->object.position.x < currentXTile*TILE_WIDTH_BYTES)
                    {                           
                        enemy->object.velocity.x = 0;
                        endOfX = TRUE;
                    }
                }
                else if(xDirection == DIR_RIGHT)
                { 
                    //this might need to be + TILE_WIDTH_BYTES
                    if(enemy->object.position.x > currentXTile*TILE_WIDTH_BYTES)
                    {                
                        enemy->object.velocity.x = 0;
                        endOfX = TRUE;
                    }
                } 
                else if(xDirection == DIR_STAND)
                {
                    enemy->object.velocity.x = 0;
                    endOfX = TRUE;
                }
                
                if(yDirection == DIR_UP)
                {
                    if(enemy->object.position.y < currentYTile*TILE_HEIGHT)
                    {
                        enemy->object.velocity.y = 0;
                        endOfY = TRUE;
                    }
                }
                else if(yDirection == DIR_DOWN)
                {
                    //this might need to be + TILE_HEIGHT
                    if(enemy->object.position.y > currentYTile*TILE_HEIGHT)
                    {
                        enemy->object.velocity.y = 0;
                        endOfY = TRUE;
                    }
                }
                else if(yDirection == DIR_STAND)
                {
                    enemy->object.velocity.y = 0;
                    endOfY = TRUE;
                }
                
                if(endOfX && endOfY)
                {
                    endOfX = FALSE;
                    endOfY = FALSE;
                    enemy->currentStep -= 2;  
                }
            }
            else
            {
                enemy->pathTravelDirection = PATH_FORWARDS;
            }

        }

        //enemyX within left boundry of shipX, enemyX within right boundry of shipX
        if(enemy->object.position.x + enemy->object.width + ENEMY_SIGHT_RANGE_ > shipObject->position.x + tileScrollAdjustment &&
            enemy->object.position.x - ENEMY_SIGHT_RANGE_ < shipObject->position.x + shipObject->width + tileScrollAdjustment)
        {
            //enemyY within top boundry of shipY, enemyY within bottom boundry of shipY 
            if(enemy->object.position.y + enemy->object.height + ENEMY_SIGHT_RANGE_*2 > shipObject->position.y &&
            enemy->object.position.y - ENEMY_SIGHT_RANGE_*2 < shipObject->position.y + shipObject->height)
            {
                enemyInRange = TRUE;
            }
        }
        else
        {
            enemyInRange = FALSE;
        }

        //shoot at the player when in range
        if(enemyInRange)
        {
            shootEnemy(enemy);
        }
    }
}

void shootEnemy(Enemy* enemy)
{
    if(enemyProjectileTimerReady)
    {
        Projectile projectile;
        projectile.object.position.x = enemy->object.position.x + enemy->object.width/2;
        projectile.object.position.y = enemy->object.position.y + enemy->object.height/2;
        projectile.direction = LEFT;
        projectile.parent = ENEMYPROJECTILE;

        initProjectile(&projectile);
        
        addListItemCopy(&projectileList, &projectile, sizeof(Projectile)); 
        enemyProjectileTimerReady = FALSE;
    }    
}

void clearEnemy(Enemy* enemy)
{
    u8* enemyVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, enemy->object.position.x, enemy->object.position.y);
    cpct_drawSolidBox(enemyVideoMemory, backgroundColour, enemy->object.width, enemy->object.height);

    //remove any of the trail that might be showing
    enemyVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, enemy->previousPosition.x, enemy->previousPosition.y);
    cpct_drawSolidBox(enemyVideoMemory, backgroundColour, enemy->object.width, enemy->object.height);
}