#include "Game.h"

u8 keys[5];
u8 worldX, worldY;

u8 gameRunning_;


const u8 exitText_[10] = {"Exit? Y/N\0"};

#define EXIT_DIALOG_WIDTH_ 38
#define EXIT_DIALOG_HEIGHT_ 20
#define EXIT_DIALOG_X_ 22

u8 confirmQuit();

void initGame(const u8 selectedLevel)
{
    //worldxy are equal to screenxy until we implement scrolling
    worldX = SCREEN_X;
    worldY = SCREEN_Y; 

    //initial map scenary
    initMap(selectedLevel);

    initShip();

    keys[LEFT] = FALSE;
    keys[RIGHT] = FALSE;
    keys[UP] = FALSE;
    keys[DOWN] = FALSE;
    keys[SHOOT] = FALSE;

    //interrupt fun
    cpct_setInterruptHandler(timerInterruptHandler);

    //setup hud
    initHud(&gameHud);

    gameRunning_ = TRUE;
}

void startGame()
{
    //reset screen position	
	cpct_setVideoMemoryOffset(0);
	
    xScrollOffset = 0;
    tileScrollX = 0;

    gameResult = ACTIVE;

    //set background and border colour
    cpct_setBorder(borderColour);
    cpct_clearScreen(backgroundColour);

    //game loop
    while (gameRunning_)
    {
        checkKeyboard();

        gameLogic();

        cpct_waitVSYNC();

        drawGame();
    }

    //destroy game after finishing
    destroyGame();
}

void checkKeyboard()
{
    cpct_scanKeyboard_f();

    if(cpct_isKeyPressed(controlsArray[CONTROLS_UP]))
    {
        keys[UP] = TRUE;       
    }
    else
    {
        keys[UP] = FALSE;
    }

    if(cpct_isKeyPressed(controlsArray[CONTROLS_DOWN]))
    {
        keys[DOWN] = TRUE;
    }
    else
    {
        keys[DOWN] = FALSE;
    }

    if(cpct_isKeyPressed(controlsArray[CONTROLS_LEFT]))
    {
        keys[LEFT] = TRUE;
    }
    else
    {
        keys[LEFT] = FALSE;
    }

    if(cpct_isKeyPressed(controlsArray[CONTROLS_RIGHT]))
    {
        keys[RIGHT] = TRUE;
    }
    else
    {
        keys[RIGHT] = FALSE;
    }

    if(cpct_isKeyPressed(controlsArray[CONTROLS_SHOOT]))
    {
        keys[SHOOT] = TRUE;
    }
    else
    {
        keys[SHOOT] = FALSE;
    }

    if(cpct_isKeyPressed(Key_Esc))
    {
        if(confirmQuit())
        {
            gameRunning_ = FALSE;
            gameResult = QUIT; 
        }
        else
        {
            gameRunning_ = TRUE;
        }
    }
}

void gameLogic()
{    
    Projectile* currentProjectile = NULL;
    Enemy* currentEnemy = NULL;
    PowerUp* currentPowerUp = NULL;
    PowerWell* currentPowerwell = NULL;
    Node* currentNode = NULL; 
//    u8 finishedScrolling = FALSE;   

    if(keys[UP])
    {
       moveShip(UP);
    }
    
    if(keys[DOWN])
    {
        moveShip(DOWN);
    }
    
    if(keys[LEFT])
    {
        moveShip(LEFT);        
    }
    
    if(keys[RIGHT])
    {
        moveShip(RIGHT);         

                 
    }

    if(keys[SHOOT])
    {
        //ship shoots a projectile             
        playerShip.shootProjectile(&playerShip);
    }

     


    //enemy logic
    currentNode = enemyList;
    
    while(currentNode != NULL)
    {
        currentEnemy = currentNode->data; 
        logicEnemy(currentEnemy, &playerShip.object);
        
		if (currentEnemy->destroyed)
		{
			clearEnemy(currentEnemy);
			removeListItemCopy(&enemyList, currentEnemy);
			currentNode = enemyList;
		}
		else
		{
			currentNode = currentNode->next;
		}        
    }

    //projectile logic
    currentNode = projectileList;

    while(currentNode != NULL)
    {
        currentProjectile = currentNode->data; 
        logicProjectile(currentProjectile);
        
        if(currentProjectile->destroyed)
        {
			clearProjectile(currentProjectile);
            removeListItemCopy(&projectileList, currentProjectile);
			currentNode = projectileList;
        }
		else
		{
			currentNode = currentNode->next;
		}    
    }

    //powerup logic
    currentNode = powerupList;

    while(currentNode != NULL)
    {
        currentPowerUp = currentNode->data; 
        
        if(currentPowerUp->used)
        {
            clearPowerUp(currentPowerUp);
            removeListItemCopy(&powerupList, currentPowerUp);
			currentNode = powerupList;
        }
		else
		{
            //redraw the powerup on map redraw
            if(mapRedraw)
            {
                currentPowerUp->redraw = TRUE;
            }

			currentNode = currentNode->next;
		}
    }   

    //powerwell logic
    currentNode = powerwellList;
    
    while(currentNode != NULL)
    {
        currentPowerwell = currentNode->data; 
        currentPowerwell->logicPowerWell(currentPowerwell, &playerShip.object);

        //clear the destroyed powerwell from screen
        if(currentPowerwell->destroyed)
        {
            clearPowerWell(currentPowerwell);
            removeListItemCopy(&powerwellList, currentPowerwell);
			currentNode = powerwellList;
        }
		else
		{
			currentNode = currentNode->next;
		}
    }

    //ship logic
    logicShip();

    //boss logic assuming its a boss level
    if(mapHasBoss)
    {
        logicBoss();
    }
    

    //check ship lifeCount
    if (playerShip.lifeCount == 0)
    {
        //lifecount 0 so game over
        gameRunning_ = FALSE;
        gameResult = LOSS;
    }

    //check if the player has reached the end of the level
    if(playerShip.reachedLevelEnd)
    {
        if(mapHasBoss)
        {
            //make sure the boss has been killed before the player wins
            if(!bossAlive)
            {
                //transition into next level or victory screen        
                gameRunning_ = FALSE;
                gameResult = WIN;
            }
        }
        else
        {
            //transition into next level or victory screen        
            gameRunning_ = FALSE;
            gameResult = WIN;
        }       
    }
}

void drawGame()
{      
    Projectile* currentProjectile = NULL;
    Enemy* currentEnemy = NULL;
    Node* currentNode = NULL;   
    PowerUp* currentPowerup = NULL; 
    PowerWell* currentPowerwell = NULL;

    //draw map
    drawMap();

    //draw player ship
    drawShip();

    //draw projectiles
    currentNode = projectileList;

    while(currentNode != NULL)
    {
        currentProjectile = currentNode->data; 

        drawProjectile(currentProjectile);
        currentNode = currentNode->next;        
    }
    
    //draw enemies
    currentNode = enemyList;

    while(currentNode != NULL)
    {
        currentEnemy = currentNode->data;
        drawEnemy(currentEnemy);
        currentNode = currentNode->next;
    }

    //draw powerups    
    currentNode = powerupList;

    while(currentNode != NULL)
    {
        currentPowerup = currentNode->data;
        drawPowerUp(currentPowerup);
        currentNode = currentNode->next;
    }

    //draw powerwells    
    currentNode = powerwellList;

    while(currentNode != NULL)
    {
        currentPowerwell = currentNode->data;
        drawPowerWell(currentPowerwell);
        currentNode = currentNode->next;
    }

    //draw boss on the boss map
    if(mapHasBoss)
    {
        drawBoss();
    }    

    //hud will only update if redraw is true
    gameHud.drawHud(&gameHud);
}

void destroyGame()
{
    clearList(&projectileList);
    clearList(&enemyList);
    clearList(&powerupList);
    clearList(&powerwellList);
    destroyMap();

    //set the previous life count for difficulty 2+
    previousLifeCount = playerShip.lifeCount;

    //reset screen colour
    cpct_clearScreen(0x00);
    cpct_setBorder(4);

    //reset screen position
	cpct_setVideoMemoryOffset(0);

    //corrects the ships direction if going left when destroy game is called
    if(playerShip.flippedDirection == LEFT)
    {
        cpct_hflipSpriteM0(playerShip.object.width, playerShip.object.height, G_ship);
    }
}

//confirm with the player if they want to quit the game
u8 confirmQuit()
{    
    u8* memory = cpct_getScreenPtr(CPCT_VMEM_START, EXIT_DIALOG_X_ + xScrollOffset, 40);
    cpct_drawSolidBox(memory, cpct_px2byteM0(0, 0) , EXIT_DIALOG_WIDTH_, EXIT_DIALOG_HEIGHT_ );
    
    memory = cpct_getScreenPtr(CPCT_VMEM_START, EXIT_DIALOG_X_ + xScrollOffset + 1, 45);
    cpct_drawStringM0(exitText_, memory, 1, 0);

    //break out of the loop once the user decides
    while(1)
    {
        cpct_scanKeyboard();

        if(cpct_isKeyPressed(Key_Y))
        {
            return TRUE;  
        }
        else if(cpct_isKeyPressed(Key_N))
        {            
            //clear just the dialog
            memory = cpct_getScreenPtr(CPCT_VMEM_START, EXIT_DIALOG_X_ + xScrollOffset, 40);
            cpct_drawSolidBox(memory, cpct_px2byteM0(5, 5) , EXIT_DIALOG_WIDTH_, EXIT_DIALOG_HEIGHT_ );

            //force a redraw of the map
            mapRedraw = TRUE;
            return FALSE;
        }
    }
}