#include "Hud.h"

#define HUD_X_POSITION_ 0
#define HUD_X_POSITION_TWO_ 64
#define HUD_Y_POSITION_ 192
#define HUD_WIDTH_ONE_ 64
#define HUD_WIDTH_TWO_ 16
#define HUD_HEIGHT_ 8

#define HUD_SCORE_START_ 0
#define LIFECOUNT_START_ 25
#define POWER_START_ 41
#define POWERWELL_START_ 55
#define HUD_SPACE_ 2

#define ASCII_TO_NUMBER_OFFSET_ 48

void _drawHud(Hud* hud);

Hud gameHud;
u8 scoreHud[6];
u8 lifeCountHud_[2];
u8 powerHud_[2];
u8 powerwellHud_[4];
u8 previousTileScrollX = 0;
u8 initialPowerwellCount = 0;

void initHud(Hud* hud)
{
    hud->drawHud = _drawHud;
    hud->redraw = TRUE;    

    //setup the lifecount hud
    lifeCountHud_[0] = playerShip.lifeCount + ASCII_TO_NUMBER_OFFSET_;
    lifeCountHud_[1] = '\0'; 

    //setup power hud       
    powerHud_[0] = playerShip.power + ASCII_TO_NUMBER_OFFSET_;
    powerHud_[1] = '\0';    

    //setup powerwell hud
    initialPowerwellCount = powerwellCount;

    powerwellHud_[0] = powerwellCount + ASCII_TO_NUMBER_OFFSET_;
    powerwellHud_[1] = '\\';
    powerwellHud_[2] = initialPowerwellCount + ASCII_TO_NUMBER_OFFSET_;
    powerwellHud_[3] = '\0';
}

void _drawHud(Hud* hud)
{    
    if(hud->redraw == TRUE || previousTileScrollX != tileScrollX)
    {      
        u8* hudVideoMemory_;

        //adjust drawing of hud x position for scrolling
        u8 tileScrollAdjustment = tileScrollX*TILE_WIDTH_BYTES;

        //draw the hud strip on the background in two parts
        hudVideoMemory_ = cpct_getScreenPtr(CPCT_VMEM_START, HUD_X_POSITION_+tileScrollAdjustment, HUD_Y_POSITION_);
        cpct_drawSolidBox(hudVideoMemory_, cpct_px2byteM0(0, 0), HUD_WIDTH_ONE_, HUD_HEIGHT_);

        hudVideoMemory_ = cpct_getScreenPtr(CPCT_VMEM_START, HUD_X_POSITION_TWO_+tileScrollAdjustment, HUD_Y_POSITION_);
        cpct_drawSolidBox(hudVideoMemory_, cpct_px2byteM0(0, 0), HUD_WIDTH_TWO_, HUD_HEIGHT_);

        //highscore
        convertIntToChar(hud->gameScore, scoreHud);
        hudVideoMemory_ = cpct_getScreenPtr(CPCT_VMEM_START, HUD_SCORE_START_+tileScrollAdjustment, HUD_Y_POSITION_);
        cpct_drawStringM0(scoreHud, hudVideoMemory_, 1, 0);


        //life count hud
        hudVideoMemory_ = cpct_getScreenPtr(CPCT_VMEM_START, LIFECOUNT_START_+tileScrollAdjustment, HUD_Y_POSITION_);
        cpct_drawSprite(G_hud_ship, hudVideoMemory_, HUD_LIFE_WIDTH, HUD_SPRITE_HEIGHT);
        
        hudVideoMemory_ = cpct_getScreenPtr(CPCT_VMEM_START, LIFECOUNT_START_ + HUD_LIFE_WIDTH + HUD_SPACE_ + tileScrollAdjustment, HUD_Y_POSITION_);
        lifeCountHud_[0] = playerShip.lifeCount + ASCII_TO_NUMBER_OFFSET_;
        cpct_drawStringM0(lifeCountHud_, hudVideoMemory_, 1, 0);


        //power hud
        hudVideoMemory_ = cpct_getScreenPtr(CPCT_VMEM_START, POWER_START_ + tileScrollAdjustment, HUD_Y_POSITION_);
        cpct_drawSprite(G_hud_power, hudVideoMemory_, HUD_POWER_WIDTH, HUD_SPRITE_HEIGHT);
        
        hudVideoMemory_ = cpct_getScreenPtr(CPCT_VMEM_START, POWER_START_ + HUD_POWER_WIDTH + HUD_SPACE_ + tileScrollAdjustment, HUD_Y_POSITION_);
        powerHud_[0] = playerShip.power + ASCII_TO_NUMBER_OFFSET_;
        cpct_drawStringM0(powerHud_, hudVideoMemory_, 1, 0);

        hud->redraw = FALSE;
        previousTileScrollX = tileScrollX;


        //powerwell count hud
        hudVideoMemory_ = cpct_getScreenPtr(CPCT_VMEM_START, POWERWELL_START_+tileScrollAdjustment, HUD_Y_POSITION_);
        cpct_drawSprite(G_hud_powerwell, hudVideoMemory_, HUD_POWERWELL_WIDTH, HUD_SPRITE_HEIGHT);
        
        hudVideoMemory_ = cpct_getScreenPtr(CPCT_VMEM_START, POWERWELL_START_ + HUD_POWERWELL_WIDTH + HUD_SPACE_ + tileScrollAdjustment, HUD_Y_POSITION_);
        
        powerwellHud_[0] = powerwellCount + ASCII_TO_NUMBER_OFFSET_;
        cpct_drawStringM0(powerwellHud_, hudVideoMemory_, 1, 0);
    }     
}

void addToScore(Hud* hud, u16 addedScore)
{
    hud->gameScore += addedScore;
    hud->redraw = TRUE;
}


//converts an input u16 to an array of u8 which is assumed to have 6 elements, 5 digits and \0
void convertIntToChar(u16 input, u8 output[6])
{    
    //need to add 48 to each number to actually display the correct number on screen    
    u8 toCharPosition = 0;
    u8 nextDigit = FALSE;

    if(input >= 10000)
    {                        
        output[toCharPosition] = (input / 10000) + ASCII_TO_NUMBER_OFFSET_;
        ++toCharPosition;
        input = (input % 10000);
        nextDigit = TRUE;
    }

    if(input >= 1000 || nextDigit)
    {                        
        output[toCharPosition] = (input / 1000) + ASCII_TO_NUMBER_OFFSET_;
        ++toCharPosition;
        input = (input % 1000);
        nextDigit = TRUE;
    }

    if(input >= 100 || nextDigit)
    {                             
        output[toCharPosition] = (input / 100) + ASCII_TO_NUMBER_OFFSET_;
        ++toCharPosition;
        input = (input % 100);
        nextDigit = TRUE;        
    }

    if(input >= 10 || nextDigit)
    {
        output[toCharPosition] = (input / 10) + ASCII_TO_NUMBER_OFFSET_;
        ++toCharPosition;
        input = (input % 10);
        nextDigit = TRUE;
    }
    
    if(input < 10 || nextDigit)
    {        
        output[toCharPosition] = input + ASCII_TO_NUMBER_OFFSET_;
        ++toCharPosition;
    }

    //add the null terminator
    output[toCharPosition] = '\0';
}