#include "Map.h"

//map arrays
u8* mapArray = NULL;
u8* enemySpawnsArray = NULL;
u8* powerupArray = NULL;
u8* powerwellArray = NULL;
u8* enemyPathArray = NULL;

//map booleans
u8 mapHasEnemies_ = FALSE;
u8 mapHasPowerUps_ = FALSE;
u8 mapHasPowerWells_ = FALSE;

//map lists
LinkedList* powerupList = NULL;
LinkedList* enemyList = NULL;
LinkedList* powerwellList = NULL;

//map counts
u8 powerupCount_ = 0;
u8 powerwellCount = 0;
u8 enemyCount_ = 0;
u8 mapXTileCount = 0;

i8 levelEnd = 0;

extern i8 mapShipXPosition = -1;
extern i8 mapShipYPosition = -1;

u8 selectedPalette = 0;

#define POWERUP_SCORE_ 100
#define POWERWELL_SCORE_ 100

u8 mapRedraw = TRUE;

void initMap(const u8 selectedLevel)
{    
    powerupCount_ = 0;
    powerwellCount = 0;
    enemyCount_ = 0;
    levelEnd = -1;
    mapShipXPosition = -1;
    mapShipYPosition = -1;

    chooseMap(selectedLevel);

    if(mapHasEnemies_)
    {
        initEnemies();
    }
    
    if(mapHasPowerUps_)
    {
        initPowerUps();
    }

    if(mapHasPowerWells_)
    {
        initPowerWells();
    }

    if(mapHasBoss)
    {
        initBoss();
    }

    //load the palette for the map
    switch(selectedPalette)
    {
        case CAVE_PALETTE_NO:
        {
            cpct_setPalette(CAVE_PALETTE, PALETTE_COLOUR_COUNT);
        }
        break;

        case UNDERWATER_PALETTE_NO:
        {
            cpct_setPalette(UNDERWATER_PALETTE, PALETTE_COLOUR_COUNT);
        }
        break;

        case CITY_PALETTE_NO:
        {
            cpct_setPalette(CITY_PALETTE, PALETTE_COLOUR_COUNT);
        }
        break;
    }
    

    mapRedraw = TRUE;
}

void drawMap()
{
    if(mapRedraw)
    {                
        u8* mapMemory;
        u8 i = 0;
        u8 j = 0;
        u16 position = 0;
        u8 yOffset = 0;

        //the purpose of this is to return 0 for every 20 or to return 21,41,61 etc as we scroll further along
        i8 currentTileScrollX = (tileScrollX % 20 != 0) * (20 * ((tileScrollX / 20) + 0) + 1);
        u8 screenedTileScrollX = tileScrollX;

        //how many screens that have been scrolled past
        u8 screens = tileScrollX / 20;

        //this is a tilescrollX that has been adjusted so that it only provides values for one screen
        if (tileScrollX >= 20)
        {
            screenedTileScrollX %= 20;
        }

        //position is the element in the map tile array
        for (i = 0; i < 20; ++i)
        {
            if (currentTileScrollX > 0 && currentTileScrollX <= tileScrollX)
            {
                position = SCREEN_TILES_Y * ((20 - 1) + currentTileScrollX);
                ++currentTileScrollX;
            }
            else if (currentTileScrollX == 0 || currentTileScrollX > tileScrollX)
            {
                position = SCREEN_TILES_Y * tileScrollX;
                currentTileScrollX = -1;
            }

            if (i <= screenedTileScrollX)
            {
                yOffset = 8;

                if (screens)
                {
                    yOffset = 8 + screens * 8;
                }
            }
            else
            {
                yOffset = 0;

                if (screens)
                {
                    yOffset = 8 * screens;
                }
            }

            //this corrects the position of the tiles on the first column of the screen
            if (i == tileScrollX % 20)
            {
                yOffset -= 8;
            }

            //this is for clearing the columns as we scroll the map so previous tiles are not left over after a scroll
            if (currentTileScrollX >= tileScrollX)
            {
                mapMemory = cpct_getScreenPtr(CPCT_VMEM_START, xPositions[i], 0);
                cpct_drawSolidBox(mapMemory, backgroundColour, 4, 16 * 12);
            }
            else
            {
                //this removes the appendage that can be seen on maxl (the 32 and 16*10 might need to be adjusted)
                if (i == 19)
                {
                    mapMemory = cpct_getScreenPtr(CPCT_VMEM_START, xPositions[i], yOffset);
                    cpct_drawSolidBox(mapMemory, backgroundColour, 4, 16 * 12);
                }
            }

            for (j = 0; j < 12; ++j)
            {
                if (mapArray[position] != 255)
                {
                    mapMemory = cpct_getScreenPtr(CPCT_VMEM_START, xPositions[i], yPositions[j] + yOffset);
                    cpct_drawSprite(all_tileset[mapArray[position]], mapMemory, 4, 16);
                }

                ++position;
            }
        }

        mapRedraw = FALSE;
    }
}

void chooseMap(const u8 selectedLevel)
{
    switch(selectedLevel)
    {
        case CAVE01:
        {
            #ifdef CAVE01_ENEMY_COUNT  
                enemySpawnsArray = enemy_spawns_cave01;     
                mapHasEnemies_ = TRUE;
                enemyCount_ = CAVE01_ENEMY_COUNT;
            #else
                mapHasEnemies_ = FALSE;
            #endif

            #ifdef CAVE01_POWERUP_COUNT
                mapHasPowerUps_ = TRUE;
                powerupCount_ = CAVE01_POWERUP_COUNT;
                powerupArray = powerups_cave01;
            #endif

            #ifdef CAVE01_POWERWELL_COUNT
                mapHasPowerWells_ = TRUE;
                powerwellCount = CAVE01_POWERWELL_COUNT;
                powerwellArray = powerwells_cave01;
            #endif

            mapXTileCount = CAVE01_X_TILE_COUNT;
            mapArray = map_cave01;
            backgroundColour = cpct_px2byteM0(CAVE01_BACKGROUND_COLOUR_1, CAVE01_BACKGROUND_COLOUR_2);
            borderColour = CAVE01_BORDER_COLOUR;
            selectedPalette = CAVE01_PALETTE;
            levelEnd = CAVE01_LEVEL_END;
            mapShipXPosition = CAVE01_SHIP_X;
            mapShipYPosition = CAVE01_SHIP_Y;
        }
        break;

        case CAVE02:
        {
            #ifdef CAVE02_ENEMY_COUNT  
                enemySpawnsArray = enemy_spawns_cave02;     
                mapHasEnemies_ = TRUE;
                enemyCount_ = CAVE02_ENEMY_COUNT;
            #else
                mapHasEnemies_ = FALSE;
            #endif

            #ifdef CAVE02_POWERUP_COUNT
                mapHasPowerUps_ = TRUE;
                powerupCount_ = CAVE02_POWERUP_COUNT;
                powerupArray = powerups_cave02;
            #endif

            #ifdef CAVE02_POWERWELL_COUNT
                mapHasPowerWells_ = TRUE;
                powerwellCount = CAVE02_POWERWELL_COUNT;
                powerwellArray = powerwells_cave02;
            #endif

            mapXTileCount = CAVE02_X_TILE_COUNT;
            mapArray = map_cave02;
            backgroundColour = cpct_px2byteM0(CAVE02_BACKGROUND_COLOUR_1, CAVE02_BACKGROUND_COLOUR_2);
            borderColour = CAVE02_BORDER_COLOUR;
            selectedPalette = CAVE02_PALETTE;
            levelEnd = CAVE02_LEVEL_END;
            mapShipXPosition = CAVE02_SHIP_X;
            mapShipYPosition = CAVE02_SHIP_Y;
        }
        break;

        case CAVE03:
        {
            #ifdef CAVE03_ENEMY_COUNT  
                enemySpawnsArray = enemy_spawns_cave03;     
                mapHasEnemies_ = TRUE;
                enemyCount_ = CAVE03_ENEMY_COUNT;
            #else
                mapHasEnemies_ = FALSE;
            #endif

            #ifdef CAVE03_POWERUP_COUNT
                mapHasPowerUps_ = TRUE;
                powerupCount_ = CAVE03_POWERUP_COUNT;
                powerupArray = powerups_cave03;
            #endif

            #ifdef CAVE03_POWERWELL_COUNT
                mapHasPowerWells_ = TRUE;
                powerwellCount = CAVE03_POWERWELL_COUNT;
                powerwellArray = powerwells_cave03;
            #endif

            mapXTileCount = CAVE03_X_TILE_COUNT;
            mapArray = map_cave03;
            backgroundColour = cpct_px2byteM0(CAVE03_BACKGROUND_COLOUR_1, CAVE03_BACKGROUND_COLOUR_2);
            borderColour = CAVE03_BORDER_COLOUR;
            selectedPalette = CAVE03_PALETTE;
            levelEnd = CAVE03_LEVEL_END;
            mapShipXPosition = CAVE03_SHIP_X;
            mapShipYPosition = CAVE03_SHIP_Y;
        }
        break;

        case CAVE04:
        {
            #ifdef CAVE04_ENEMY_COUNT  
                enemySpawnsArray = enemy_spawns_cave04;     
                mapHasEnemies_ = TRUE;
                enemyCount_ = CAVE04_ENEMY_COUNT;
            #else
                mapHasEnemies_ = FALSE;
            #endif

            #ifdef CAVE04_POWERUP_COUNT
                mapHasPowerUps_ = TRUE;
                powerupCount_ = CAVE04_POWERUP_COUNT;
                powerupArray = powerups_cave04;
            #endif

            #ifdef CAVE04_POWERWELL_COUNT
                mapHasPowerWells_ = TRUE;
                powerwellCount = CAVE04_POWERWELL_COUNT;
                powerwellArray = powerwells_cave04;
            #endif

            mapXTileCount = CAVE04_X_TILE_COUNT;
            mapArray = map_cave04;
            backgroundColour = cpct_px2byteM0(CAVE04_BACKGROUND_COLOUR_1, CAVE04_BACKGROUND_COLOUR_2);
            borderColour = CAVE04_BORDER_COLOUR;
            selectedPalette = CAVE04_PALETTE;
            levelEnd = CAVE04_LEVEL_END;
            mapShipXPosition = CAVE04_SHIP_X;
            mapShipYPosition = CAVE04_SHIP_Y;
        }
        break;

        case UNDERWATER01:
        {
            #ifdef UNDERWATER01_ENEMY_COUNT  
                enemySpawnsArray = enemy_spawns_underwater01;     
                mapHasEnemies_ = TRUE;
                enemyCount_ = UNDERWATER01_ENEMY_COUNT;
                enemyPathArray = enemy_spawns_paths_underwater01;
            #else
                mapHasEnemies_ = FALSE;
            #endif

            #ifdef UNDERWATER01_POWERUP_COUNT
                mapHasPowerUps_ = TRUE;
                powerupCount_ = UNDERWATER01_POWERUP_COUNT;
                powerupArray = powerups_underwater01;
            #endif

            #ifdef UNDERWATER01_POWERWELL_COUNT
                mapHasPowerWells_ = TRUE;
                powerwellCount = UNDERWATER01_POWERWELL_COUNT;
                powerwellArray = powerwells_underwater01;
            #endif

            mapXTileCount = UNDERWATER01_X_TILE_COUNT;
            mapArray = map_underwater01;
            backgroundColour = cpct_px2byteM0(UNDERWATER01_BACKGROUND_COLOUR_1, UNDERWATER01_BACKGROUND_COLOUR_2);
            borderColour = UNDERWATER01_BORDER_COLOUR;
            selectedPalette = UNDERWATER01_PALETTE;
            levelEnd = UNDERWATER01_LEVEL_END;
            mapShipXPosition = UNDERWATER01_SHIP_X;
            mapShipYPosition = UNDERWATER01_SHIP_Y;
        }
        break;

        case UNDERWATER02:
        {
            #ifdef UNDERWATER02_ENEMY_COUNT  
                enemySpawnsArray = enemy_spawns_underwater02;     
                mapHasEnemies_ = TRUE;
                enemyCount_ = UNDERWATER02_ENEMY_COUNT;
                enemyPathArray = enemy_spawns_paths_underwater02;
            #else
                mapHasEnemies_ = FALSE;
            #endif

            #ifdef UNDERWATER02_POWERUP_COUNT
                mapHasPowerUps_ = TRUE;
                powerupCount_ = UNDERWATER02_POWERUP_COUNT;
                powerupArray = powerups_underwater02;
            #endif

            #ifdef UNDERWATER02_POWERWELL_COUNT
                mapHasPowerWells_ = TRUE;
                powerwellCount = UNDERWATER02_POWERWELL_COUNT;
                powerwellArray = powerwells_underwater02;
            #endif

            mapXTileCount = UNDERWATER02_X_TILE_COUNT;
            mapArray = map_underwater02;
            backgroundColour = cpct_px2byteM0(UNDERWATER02_BACKGROUND_COLOUR_1, UNDERWATER02_BACKGROUND_COLOUR_2);
            borderColour = UNDERWATER02_BORDER_COLOUR;
            selectedPalette = UNDERWATER02_PALETTE;
            levelEnd = UNDERWATER02_LEVEL_END;
            mapShipXPosition = UNDERWATER02_SHIP_X;
            mapShipYPosition = UNDERWATER02_SHIP_Y;
        }
        break;

        case UNDERWATER03:
        {
            #ifdef UNDERWATER03_ENEMY_COUNT  
                enemySpawnsArray = enemy_spawns_underwater03;     
                mapHasEnemies_ = TRUE;
                enemyCount_ = UNDERWATER03_ENEMY_COUNT;
                enemyPathArray = enemy_spawns_paths_underwater03;
            #else
                mapHasEnemies_ = FALSE;
            #endif

            #ifdef UNDERWATER03_POWERUP_COUNT
                mapHasPowerUps_ = TRUE;
                powerupCount_ = UNDERWATER03_POWERUP_COUNT;
                powerupArray = powerups_underwater03;
            #endif

            #ifdef UNDERWATER03_POWERWELL_COUNT
                mapHasPowerWells_ = TRUE;
                powerwellCount = UNDERWATER03_POWERWELL_COUNT;
                powerwellArray = powerwells_underwater03;
            #endif

            mapXTileCount = UNDERWATER03_X_TILE_COUNT;
            mapArray = map_underwater03;
            backgroundColour = cpct_px2byteM0(UNDERWATER03_BACKGROUND_COLOUR_1, UNDERWATER03_BACKGROUND_COLOUR_2);
            borderColour = UNDERWATER03_BORDER_COLOUR;
            selectedPalette = UNDERWATER03_PALETTE;
            levelEnd = UNDERWATER03_LEVEL_END;
            mapShipXPosition = UNDERWATER03_SHIP_X;
            mapShipYPosition = UNDERWATER03_SHIP_Y;
        }
        break;

        case UNDERWATER04:
        {
            #ifdef UNDERWATER04_ENEMY_COUNT  
                enemySpawnsArray = enemy_spawns_underwater04;     
                mapHasEnemies_ = TRUE;
                enemyCount_ = UNDERWATER04_ENEMY_COUNT;
                enemyPathArray = enemy_spawns_paths_underwater04;
            #else
                mapHasEnemies_ = FALSE;
            #endif

            #ifdef UNDERWATER04_POWERUP_COUNT
                mapHasPowerUps_ = TRUE;
                powerupCount_ = UNDERWATER04_POWERUP_COUNT;
                powerupArray = powerups_underwater04;
            #endif

            #ifdef UNDERWATER04_POWERWELL_COUNT
                mapHasPowerWells_ = TRUE;
                powerwellCount = UNDERWATER04_POWERWELL_COUNT;
                powerwellArray = powerwells_underwater04;
            #endif

            mapXTileCount = UNDERWATER04_X_TILE_COUNT;
            mapArray = map_underwater04;
            backgroundColour = cpct_px2byteM0(UNDERWATER04_BACKGROUND_COLOUR_1, UNDERWATER04_BACKGROUND_COLOUR_2);
            borderColour = UNDERWATER04_BORDER_COLOUR;
            selectedPalette = UNDERWATER04_PALETTE;
            levelEnd = UNDERWATER04_LEVEL_END;
            mapShipXPosition = UNDERWATER04_SHIP_X;
            mapShipYPosition = UNDERWATER04_SHIP_Y;
        }
        break;

        case CITY01:
        {
            #ifdef CITY01_ENEMY_COUNT  
                enemySpawnsArray = enemy_spawns_city01;     
                mapHasEnemies_ = TRUE;
                enemyCount_ = CITY01_ENEMY_COUNT;
                enemyPathArray = enemy_spawns_paths_city01;
            #else
                mapHasEnemies_ = FALSE;
            #endif

            #ifdef CITY01_POWERUP_COUNT
                mapHasPowerUps_ = TRUE;
                powerupCount_ = CITY01_POWERUP_COUNT;
                powerupArray = powerups_city01;
            #endif

            #ifdef CITY01_POWERWELL_COUNT
                mapHasPowerWells_ = TRUE;
                powerwellCount = CITY01_POWERWELL_COUNT;
                powerwellArray = powerwells_city01;
            #endif

            mapXTileCount = CITY01_X_TILE_COUNT;
            mapArray = map_city01;
            backgroundColour = cpct_px2byteM0(CITY01_BACKGROUND_COLOUR_1, CITY01_BACKGROUND_COLOUR_2);
            borderColour = CITY01_BORDER_COLOUR;
            selectedPalette = CITY01_PALETTE;
            levelEnd = CITY01_LEVEL_END;
            mapShipXPosition = CITY01_SHIP_X;
            mapShipYPosition = CITY01_SHIP_Y;
        }
        break;

        case CITY02:
        {
            #ifdef CITY02_ENEMY_COUNT  
                enemySpawnsArray = enemy_spawns_city02;     
                mapHasEnemies_ = TRUE;
                enemyCount_ = CITY01_ENEMY_COUNT;
                enemyPathArray = enemy_spawns_paths_city02;
            #else
                mapHasEnemies_ = FALSE;
            #endif

            #ifdef CITY02_POWERUP_COUNT
                mapHasPowerUps_ = TRUE;
                powerupCount_ = CITY02_POWERUP_COUNT;
                powerupArray = powerups_city02;
            #endif

            #ifdef CITY02_POWERWELL_COUNT
                mapHasPowerWells_ = TRUE;
                powerwellCount = CITY02_POWERWELL_COUNT;
                powerwellArray = powerwells_city02;
            #endif

            mapXTileCount = CITY02_X_TILE_COUNT;
            mapArray = map_city02;
            backgroundColour = cpct_px2byteM0(CITY02_BACKGROUND_COLOUR_1, CITY02_BACKGROUND_COLOUR_2);
            borderColour = CITY02_BORDER_COLOUR;
            selectedPalette = CITY02_PALETTE;
            levelEnd = CITY02_LEVEL_END;
            mapShipXPosition = CITY02_SHIP_X;
            mapShipYPosition = CITY02_SHIP_Y;
        }
        break;

        case CITY03:
        {
            #ifdef CITY03_ENEMY_COUNT  
                enemySpawnsArray = enemy_spawns_city03;     
                mapHasEnemies_ = TRUE;
                enemyCount_ = CITY03_ENEMY_COUNT;
                enemyPathArray = enemy_spawns_paths_city03;
            #else
                mapHasEnemies_ = FALSE;
            #endif

            #ifdef CITY03_POWERUP_COUNT
                mapHasPowerUps_ = TRUE;
                powerupCount_ = CITY03_POWERUP_COUNT;
                powerupArray = powerups_city03;
            #endif

            #ifdef CITY03_POWERWELL_COUNT
                mapHasPowerWells_ = TRUE;
                powerwellCount = CITY03_POWERWELL_COUNT;
                powerwellArray = powerwells_city03;
            #endif

            mapXTileCount = CITY03_X_TILE_COUNT;
            mapArray = map_city03;
            backgroundColour = cpct_px2byteM0(CITY03_BACKGROUND_COLOUR_1, CITY03_BACKGROUND_COLOUR_2);
            borderColour = CITY03_BORDER_COLOUR;
            selectedPalette = CITY03_PALETTE;
            levelEnd = CITY03_LEVEL_END;
            mapShipXPosition = CITY03_SHIP_X;
            mapShipYPosition = CITY03_SHIP_Y;
        }
        break;

        case CITY04:
        {
            #ifdef CITY04_ENEMY_COUNT  
                enemySpawnsArray = enemy_spawns_city04;     
                mapHasEnemies_ = TRUE;
                enemyCount_ = CITY04_ENEMY_COUNT;
            #else
                mapHasEnemies_ = FALSE;
            #endif

            #ifdef CITY04_POWERUP_COUNT
                mapHasPowerUps_ = TRUE;
                powerupCount_ = CITY04_POWERUP_COUNT;
                powerupArray = powerups_city04;
            #endif

            #ifdef CITY04_POWERWELL_COUNT
                mapHasPowerWells_ = TRUE;
                powerwellCount = CITY04_POWERWELL_COUNT;
                powerwellArray = powerwells_city04;
            #endif

            mapXTileCount = CITY04_X_TILE_COUNT;
            mapArray = map_city04;
            backgroundColour = cpct_px2byteM0(CITY04_BACKGROUND_COLOUR_1, CITY04_BACKGROUND_COLOUR_2);
            borderColour = CITY04_BORDER_COLOUR;
            selectedPalette = CITY04_PALETTE;
            levelEnd = CITY04_LEVEL_END;
            mapShipXPosition = CITY04_SHIP_X;
            mapShipYPosition = CITY04_SHIP_Y;
            bossPosition.x = CITY04_BOSS_X * TILE_WIDTH_BYTES;
            bossPosition.y = CITY04_BOSS_Y * TILE_HEIGHT;
            mapHasBoss = TRUE;
        }
        break;
    } 
}

void initEnemies()
{   
    u8 i, j;

    for(i = 0; i<ENEMY_COMPONENT_COUNT*enemyCount_; i+=ENEMY_COMPONENT_COUNT)
    {
        //create an enemy for each enemy spawn count       
        for(j = 0; j<enemySpawnsArray[i+3]; ++j)
        {
            Enemy newEnemy;
            u16 x, y;
            //i = x
            //i+1 = y
            //i+2 = type
            //i+3 = spawn count TODO
            //i+4 = enemypath array start position
            //i+5 = enemy path step count (this includes both x and y so half of the actual array element count)
            //i+6 = enemy sprite number
            //ignoring count for now
            x = enemySpawnsArray[i];
            x = x << 2;
            
            y = enemySpawnsArray[i+1];
            y = y << 4;

            //initialise the test enemy and add it to a list
            initEnemy(&newEnemy, enemySpawnsArray[i+2], x, y, ENEMY_WIDTH_BYTES, ENEMY_HEIGHT);
            newEnemy.path = enemyPathArray+(enemySpawnsArray[i+4]);
            newEnemy.pathSteps = enemySpawnsArray[i+5];
            newEnemy.currentStep = 0;
            newEnemy.spriteNumber = enemySpawnsArray[i+6];

            //turret = 100 points
            //repeating flying = 200 points
            //file and off = 300 points
            newEnemy.score = (newEnemy.type + 1) * 100;

            //set the time to wait before an enemy can be spawned
            newEnemy.enemySpawnTime = j;

            addListItemCopy(&enemyList, &newEnemy, sizeof(Enemy));
        }       
    }  
}

void initPowerUps()
{
    PowerUp newPowerup;
    u8 i = 0;

    for(i = 0; i<powerupCount_*POWERUP_COMPONENT_COUNT; i+=POWERUP_COMPONENT_COUNT)
    {
        newPowerup.object.position.x = powerupArray[i]*TILE_WIDTH_BYTES;
        newPowerup.object.position.y = powerupArray[i+1]*TILE_HEIGHT;
        newPowerup.object.width = POWERUP_WIDTH_BYTES;
        newPowerup.object.height = POWERUP_HEIGHT;
        newPowerup.type = powerupArray[i+2];
        newPowerup.used = FALSE;
        newPowerup.redraw = TRUE;

        //all powerwells are currently considered equal
        newPowerup.score = POWERUP_SCORE_;

        addListItemCopy(&powerupList, &newPowerup, sizeof(PowerUp));
    }
}

void initPowerWells()
{
    PowerWell newPowerwell;
    u8 i = 0;

    for(i = 0; i<powerwellCount*POWERWELL_COMPONENT_COUNT; i+=POWERWELL_COMPONENT_COUNT)
    {
        initPowerWell(&newPowerwell, powerwellArray[i]*TILE_WIDTH_BYTES, powerwellArray[i+1]*TILE_HEIGHT, TILE_WIDTH_BYTES, TILE_HEIGHT*powerwellArray[i+2], powerwellArray[i+3]);
        
        //all powerwells are currently considered equal
        newPowerwell.score = POWERWELL_SCORE_;
        
        addListItemCopy(&powerwellList, &newPowerwell, sizeof(PowerWell));
    }
}

void destroyMap()
{
    mapHasEnemies_ = FALSE;
    mapHasPowerUps_ = FALSE;
    mapHasPowerWells_ = FALSE;
    mapArray = NULL; 
    powerupArray = NULL;
    powerwellArray = NULL;
    enemySpawnsArray = NULL;
    mapHasBoss = FALSE;
}