#include "Menu.h"

#define ITEM_WIDTH 64
#define ITEM_HEIGHT 9
#define MENU_SPACE 25
#define LEFT_MARGIN 2

void runSelectedMenuItem(Menu* menu, u8 itemNumber);

void initMenu(Menu* menu, MenuItem* menuItems, u8 menuItemCount)
{
    menu->currentMenuItemNo = 0;
    menu->menuItemCount = menuItemCount;
    menu->menuItems = menuItems;
    menu->running = FALSE;
    menu->changed = TRUE;
}

void drawMenu(Menu* menu)
{
    u8* menuMemory = NULL;
    u8 i = 0;

    //draw menu title
    menuMemory = cpct_getScreenPtr(CPCT_VMEM_START, LEFT_MARGIN, 5);      
    cpct_drawStringM0(menu->menuTitle,  menuMemory, 1, 0);

    //draw all the menu items
    for(i = 0 ; i<menu->menuItemCount; ++i)    
    {        
        MenuItem* menuItem = (menu->menuItems + i);
        menuItem->drawMenuItem(menuItem, LEFT_MARGIN, MENU_SPACE + i*ITEM_HEIGHT, ITEM_WIDTH, ITEM_HEIGHT, i == menu->currentMenuItemNo);
    }    
}

void runMenu(Menu* menu)
{
    do
    {
        cpct_scanKeyboard();
    }
    while(cpct_isKeyPressed(Key_Enter) || cpct_isKeyPressed(Key_Return) || cpct_isKeyPressed(Key_Space));

    cpct_clearScreen(0x00);

    menu->running = TRUE;
    menu->changed = TRUE;

    while(menu->running)
    {        
        readKeyboard(menu);
        if(menu->changed)
        {
            drawMenu(menu);
            menu->changed = FALSE;
        }
        
        cpct_waitVSYNC();
    }    
}

void readKeyboard(Menu* menu)
{
    cpct_scanKeyboard_f();

    //up and down to change the selected menu item
    if(cpct_isKeyPressed(Key_CursorUp))
    {
        if(menu->currentMenuItemNo > 0)
        {
            --menu->currentMenuItemNo;
            menu->changed = TRUE;
        }
    }    
    else if(cpct_isKeyPressed(Key_CursorDown))
    {
        if(menu->currentMenuItemNo + 1 < menu->menuItemCount)
        {
            ++menu->currentMenuItemNo;
            menu->changed = TRUE;
        }
    }

    //run one of the numbered menu items
    if(cpct_isKeyPressed(Key_1))
    {
        runSelectedMenuItem(menu, 0);
    }
    else if(cpct_isKeyPressed(Key_2))
    {
        runSelectedMenuItem(menu, 1);
    }
    else if(cpct_isKeyPressed(Key_3))
    {
        runSelectedMenuItem(menu, 2);
    }
    else if(cpct_isKeyPressed(Key_4))
    {
        runSelectedMenuItem(menu, 3);
    }
    else if(cpct_isKeyPressed(Key_5))
    {
        runSelectedMenuItem(menu, 4); 
    }
    else if(cpct_isKeyPressed(Key_6))
    {
        runSelectedMenuItem(menu, 5);
    }
    else if(cpct_isKeyPressed(Key_7))
    {
        runSelectedMenuItem(menu, 6);
    }
    else if(cpct_isKeyPressed(Key_8))
    {
        runSelectedMenuItem(menu, 7);
    }
    else if(cpct_isKeyPressed(Key_9))
    {
        runSelectedMenuItem(menu, 8);
    }

    //run the menu item that is currently selected
    if(cpct_isKeyPressed(Key_Enter) || cpct_isKeyPressed(Key_Return) || cpct_isKeyPressed(Key_Space))
    {
        runSelectedMenuItem(menu, menu->currentMenuItemNo);       
    }
}

void runSelectedMenuItem(Menu* menu, u8 itemNumber)
{
    if(itemNumber < menu->menuItemCount)
    {
        MenuItem* menuItem = (menu->menuItems + itemNumber);
        menuItem->runMenuItem();
        menu->running = FALSE;
    }    
}

void exitMenu(Menu* menu)
{
    menu->running = FALSE;
}