#include "PowerWell.h"

#define POWERWELL_SITE_RANGE_ 5
#define POWERWELL_BASE_HEIGHT_ 8

void _logicPowerWell(PowerWell* powerwell, Object* playerObject);

void initPowerWell(PowerWell* powerwell, u16 x, u16 y, u8 width, u8 height, PowerWellType type)
{
    powerwell->object.position.x = x;
    powerwell->object.position.y = y;
    powerwell->object.width = width;
    powerwell->object.height = height;
    powerwell->type = type;
    powerwell->destroyed = FALSE;
    powerwell->logicPowerWell = _logicPowerWell;
    powerwell->hitCount = 0;
    initPowerWellAttack(&powerwell->attack);     
}

void drawPowerWell(PowerWell* powerwell)
{
    u8* powerwellVideoMemory;
    u16 scrolledXPosition = powerwell->object.position.x - (tileScrollX*4);

    //make sure that the powerwell is on screen before drawing it
    if(scrolledXPosition < SCREEN_X)
    {
        //turns out powerwell.object.position.x is the right place to draw
        powerwellVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, powerwell->object.position.x, powerwell->object.position.y);
        
        //different powerwell types have different colours and bases
        switch(powerwell->type)
        {
            case POSITIVE:
            {
                cpct_drawSolidBox(powerwellVideoMemory, cpct_px2byteM0(2, 4), powerwell->object.width, powerwell->object.height);
                powerwellVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, powerwell->object.position.x, powerwell->object.position.y + powerwell->object.height - POWERWELL_BASE_HEIGHT_);
                
                //draw powerwell base
                cpct_drawSprite(G_positivePowerWellBase, powerwellVideoMemory, powerwell->object.width, POWERWELL_BASE_HEIGHT_);
            }
            break;

            case NEGATIVE:
            {
                cpct_drawSolidBox(powerwellVideoMemory, cpct_px2byteM0(2, 3), powerwell->object.width, powerwell->object.height);
                powerwellVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, powerwell->object.position.x, powerwell->object.position.y + powerwell->object.height - POWERWELL_BASE_HEIGHT_);
                
                //draw powerwell base
                cpct_drawSprite(G_negativePowerWellBase, powerwellVideoMemory, powerwell->object.width, POWERWELL_BASE_HEIGHT_);
            }
            break;
        }

        if(powerwell->attack.active == TRUE)
        {
            powerwell->attack.drawAttack(&powerwell->attack);
        }
    }   
}

void clearPowerWell(PowerWell* powerwell)
{
    u8* powerwellVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, powerwell->object.position.x, powerwell->object.position.y);
    cpct_drawSolidBox(powerwellVideoMemory, backgroundColour, powerwell->object.width, powerwell->object.height);  
}

void _logicPowerWell(PowerWell* powerwell, Object* shipObject)
{
    u8 shipInRange = FALSE;
    
    //enemyX within left boundry of shipX, enemyX within right boundry of shipX
    if(powerwell->object.position.x + powerwell->object.width + POWERWELL_SITE_RANGE_ > shipObject->position.x && //RIGHT of well
        powerwell->object.position.x - POWERWELL_SITE_RANGE_ < shipObject->position.x + shipObject->width) //LEFT of well
    {
        //enemyY within top boundry of shipY, enemyY within bottom boundry of shipY 
        if(shipObject->position.y + shipObject->height >= powerwell->object.position.y &&
           shipObject->position.y < powerwell->object.position.y + powerwell->object.height)
        {
            shipInRange = TRUE;
        }
    }

    //shoot at the player when in range
    if(shipInRange)
    {
        if(shipObject->position.x + shipObject->width < powerwell->object.position.x) //left of the well
        {
            powerwell->attack.object.width = shipObject->width + POWERWELL_SITE_RANGE_;
            powerwell->attack.object.position.x = shipObject->position.x;

            powerwell->attack.active = TRUE;
            powerwell->attack.object.position.y = (shipObject->position.y >= powerwell->object.position.y ? shipObject->position.y : shipObject->position.y + shipObject->height);                
            powerwell->attack.object.height = 2;
        }
        else if(shipObject->position.x > powerwell->object.position.x + powerwell->object.width) //right of the well
        {
            powerwell->attack.object.width = POWERWELL_SITE_RANGE_;
            powerwell->attack.object.position.x = powerwell->object.position.x + powerwell->object.width;

            powerwell->attack.active = TRUE;
            powerwell->attack.object.position.y = (shipObject->position.y >= powerwell->object.position.y ? shipObject->position.y : shipObject->position.y + shipObject->height);                
            powerwell->attack.object.height = 2;
        }
    }
    else
    {
        powerwell->attack.active = FALSE;        
    }

    if(powerwell->attack.active == TRUE)
    {
        switch(powerwell->type)
        {
            case POSITIVE:
            {
                //positive wells increase speed and power
                if(playerShip.power < 5)
                {
                    ++playerShip.power;
                    gameHud.redraw = TRUE;
                }
                
                if(playerShip.shipSpeed == 1)
                {
                    playerShip.shipSpeed = 2;
                    gameHud.redraw = TRUE;
                }
            }
            break;

            case NEGATIVE:
            {
                //negative wells set speed and power to minimum
                if(playerShip.power != 1 || playerShip.shipSpeed != 1)
                {
                    playerShip.power = 1;
                    playerShip.shipSpeed = 1;
                    gameHud.redraw = TRUE;
                }
            }
            break;
        }        
    }
}