#include "Projectile.h"

void initProjectile(Projectile* projectile)
{
    projectile->object.width = 2;
    projectile->object.height = 4;

    //set initial velocity
    if(projectile->direction == RIGHT)
    {
        projectile->object.velocity.x = 2;
    }
    else
    {
        projectile->object.velocity.x = -2;
    }
    
    projectile->object.velocity.y = 0;
    projectile->previousPosition.x = projectile->object.position.x;
    projectile->previousPosition.y = projectile->object.position.y;
	projectile->destroyed = FALSE;
}

void drawProjectile(Projectile* projectile)
{
    u8* projectileVideoMemory;
    
    //adjust drawing of projectile x position for scrolling
    u8 tileScrollAdjustment = 0;//tileScrollX*2;

    //draw over the tail that results from movement
    if(projectile->previousPosition.x != projectile->object.position.x || projectile->previousPosition.y != projectile->object.position.y)
    {
        projectileVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, projectile->previousPosition.x, projectile->previousPosition.y);
        cpct_drawSolidBox(projectileVideoMemory, backgroundColour, projectile->object.width + tileScrollAdjustment, projectile->object.height);
    } 
    
    projectileVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, projectile->object.position.x + tileScrollAdjustment, projectile->object.position.y);
    cpct_drawSolidBox(projectileVideoMemory, cpct_px2byteM0(1, 1), projectile->object.width, projectile->object.height);
}

void logicProjectile(Projectile* projectile)
{
    u8 projectilexw = projectile->object.position.x + projectile->object.width;
    u8 projectileyh = projectile->object.position.y + projectile->object.height;
    u8 tileScrollAdjustment = tileScrollX*2;

    //set previous position for the projectile
    projectile->previousPosition.x = projectile->object.position.x;
    projectile->previousPosition.y = projectile->object.position.y;
    
    //calculate new projectile x position from x velocity
    if(projectile->object.velocity.x > 0)
    {
        if(projectile->object.position.x < SCREEN_X + tileScrollX*TILE_WIDTH_BYTES - projectile->object.width)
        {
            projectile->object.position.x += 1;
        }
        else
        {
            //set the projectile as destroyed once it hits the edge of the screen
			projectile->destroyed = TRUE;
        }             
    }
    else if(projectile->object.velocity.x < 0)
    {
        if(projectile->object.position.x > 0 + tileScrollX*TILE_WIDTH_BYTES)
        {
            projectile->object.position.x -= 1;
        }
        else
        {
            //set the projectile as destroyed once it hits the edge of the screen
			projectile->destroyed = TRUE;
        }     
    }

	//no point doing anything once the projectile is destroyed
	if (projectile->destroyed == FALSE)
	{
		//calculate new projectile y position from y velocity
		if (projectile->object.velocity.y > 0)
		{
			if (projectile->object.position.y < SCREEN_Y - projectile->object.height)
			{
				projectile->object.position.y += 1;
			}
			else
			{
				//projectile is removed once it hits the edge of the screen
				clearProjectile(projectile);
				removeListItemCopy(&projectileList, projectile);
			}
		}
		else if (projectile->object.velocity.y < 0)
		{
			if (projectile->object.position.y > 0)
			{
				projectile->object.position.y -= 1;
			}
			else
			{
				//projectile is removed once it hits the edge of the screen
				clearProjectile(projectile);
				removeListItemCopy(&projectileList, projectile);
			}
		}

		//check for collision with enemies    
		if (listItemCount(&enemyList) > 0 && projectile->parent == SHIPPROJECTILE)
		{
			Enemy* currentEnemy = NULL;
			Node* currentNode = enemyList;

			while (currentNode != NULL)
			{
				currentEnemy = currentNode->data;

				if (projectilexw >= currentEnemy->object.position.x &&
					projectile->object.position.x <= currentEnemy->object.position.x + currentEnemy->object.width)
				{
					if (projectileyh >= currentEnemy->object.position.y &&
						projectile->object.position.y <= currentEnemy->object.position.y + currentEnemy->object.height)
					{
						//add points
						addToScore(&gameHud, (currentEnemy->type + 1) * 100);

						//set the enemy as destroyed
						currentEnemy->destroyed = TRUE;
					}
				}

				currentNode = currentNode->next;
			}
		}

		//shooting powerwells
		if (listItemCount(&projectileList) > 0 && projectile->parent == SHIPPROJECTILE)
		{
			PowerWell* currentWell = NULL;
			Node* currentNode = powerwellList;

			while (currentNode != NULL)
			{
				currentWell = currentNode->data;

				if (projectilexw >= currentWell->object.position.x &&
					projectile->object.position.x <= currentWell->object.position.x + currentWell->object.width)
				{
					if (projectileyh >= currentWell->object.position.y &&
						projectile->object.position.y <= currentWell->object.position.y + currentWell->object.height)
					{						
						//a powerwell can take a number of hits before it is destroyed
						currentWell->hitCount += playerShip.power;

						//once it reaches the limit it will be set to destroyed
						if(currentWell->hitCount >= POWERWELL_DEFAULT_LIFE)
						{
							currentWell->destroyed = TRUE;

							//increase the score when the powerwell is destroyed
							addToScore(&gameHud, POWERWELL_SCORE);

							//decrease the powerwell count
							--powerwellCount;

							//force hud redraw
							gameHud.redraw = TRUE;
						}

						//set projectile destroyed once it hits a well
						projectile->destroyed = TRUE;
						break;
					}
				}

				currentNode = currentNode->next;
			}
		}

		//projectile collision with ship detection
		if (projectile->parent == ENEMYPROJECTILE)
		{
			if (projectilexw >= playerShip.object.position.x &&
				projectile->object.position.x <= playerShip.object.position.x + playerShip.object.width)
			{
				if (projectileyh >= playerShip.object.position.y &&
					projectile->object.position.y <= playerShip.object.position.y + playerShip.object.height)
				{
					//enemy projectile colides with enemy causes loss of life
					--playerShip.lifeCount;

					//force a redraw of the hud because of the change in lifecount
					gameHud.redraw = TRUE;

					//destroy the projectile after it meets its target
					projectile->destroyed = TRUE;
				}
			}
		}

		//ship shooting boss
		if(mapHasBoss & projectile->parent == SHIPPROJECTILE)
		{
			if (projectilexw >= bossPosition.x &&
					projectile->object.position.x <= bossPosition.x + BOSS_WIDTH)
			{
				if (projectileyh >= bossPosition.y &&
					projectile->object.position.y <= bossPosition.y + BOSS_HEIGHT)
				{
					//increase boss hitcount
					bossHits += playerShip.power;

					//destroy the projectile after it meets its target
					projectile->destroyed = TRUE;

					//once shot enough kill the boss and clear the space
					if(bossHits > 10*powerwellCount*2 + 15)
					{
						killBoss();					
					}
				}
			}
		}

	}
}

void shootProjectile(Projectile* projectile)
{
    projectile->object.velocity.x = 1;
}

void clearProjectile(Projectile* projectile)
{
    u8* projectileVideoMemory;

    projectileVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, projectile->object.position.x, projectile->object.position.y);
    cpct_drawSolidBox(projectileVideoMemory, backgroundColour, projectile->object.width, projectile->object.height);
}