#include "Ship.h"

#define DEFAULT_LIFE_COUNT_ 3
#define DEFAULT_SHIP_POWER_ 1
#define DEFAULT_SHIP_SPEED_ 1
#define MAXIMUM_LIFE_COUNT_ 9
#define MAXIMUM_SHOOT_POWER_ 5

#define MAPA(x,y) mapArray[x*SCREEN_TILES_Y + y]

Ship playerShip;
u8 positionChanged_ = FALSE;
u8 previousTileScrollX_ = 0;
u8 hitSolid_ = FALSE;
void _shootProjectile(Ship* ship);
u8 finishedScrolling = FALSE;  
u8 previousLifeCount = DEFAULT_LIFE_COUNT_;

void initShip()
{
    playerShip.object.width = SHIP_WIDTH_BYTES;
    playerShip.object.height = SHIP_HEIGHT;
    playerShip.object.position.x = 20;
    playerShip.object.position.y = 96;
    playerShip.direction = RIGHT;
    playerShip.flippedDirection = RIGHT;
    playerShip.previousPosition.x = playerShip.object.position.x;
    playerShip.previousPosition.y = playerShip.object.position.y;
    playerShip.shipSpeed = DEFAULT_SHIP_SPEED_;
    playerShip.shootProjectile = _shootProjectile;
    
    //level 1 difficulty gives you 3 lives, level 2 keeps the previous count
    playerShip.lifeCount = (difficultyLevel > 1 ? previousLifeCount : DEFAULT_LIFE_COUNT_);
    playerShip.power = DEFAULT_SHIP_POWER_;
    playerShip.reachedLevelEnd = FALSE;

    //load the ship position from the map
    if(mapShipXPosition != -1 && mapShipYPosition != -1)
    {
        playerShip.object.position.x = mapShipXPosition * TILE_WIDTH_BYTES;
        playerShip.object.position.y = mapShipYPosition * TILE_HEIGHT;
    }

    //reset scrolling
    finishedScrolling = FALSE; 
    xScrollOffset = 0;
}

void drawShip()
{    
    u8* shipVideoMemory;
    
    //draw over the previous trail (adjusting for scrolling)
    if(playerShip.previousPosition.x != playerShip.object.position.x || playerShip.previousPosition.y != playerShip.object.position.y)
    {
        //cover previous position
        shipVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, playerShip.previousPosition.x, playerShip.previousPosition.y);
        cpct_drawSolidBox(shipVideoMemory, backgroundColour, playerShip.object.width, playerShip.object.height);

        //this corrects the tails that occur after scrolling in the right direction
        //hitsolid partially fixes the issue of running into walls and remains being leftover
        if(previousTileScrollX_ != tileScrollX || hitSolid_)
        {
            shipVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, playerShip.previousPosition.x - 2, playerShip.previousPosition.y);
            cpct_drawSolidBox(shipVideoMemory, backgroundColour, 2, playerShip.object.height);
            hitSolid_ = FALSE;
        }        
    }   
 
    if(playerShip.direction != playerShip.flippedDirection)
    {
        if(playerShip.flippedDirection == LEFT)
        {
            playerShip.flippedDirection = RIGHT;
        }
        else
        {
            playerShip.flippedDirection = LEFT;
        }

        cpct_hflipSpriteM0(playerShip.object.width, playerShip.object.height, G_ship);
    }

    //draw the player ship sprite 
    shipVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, playerShip.object.position.x, playerShip.object.position.y);
    cpct_drawSprite(G_ship, shipVideoMemory, playerShip.object.width, playerShip.object.height);

    previousTileScrollX_ = tileScrollX;    
}

void logicShip()
{
    u8 xTile = 0;
    u8 xTile2 = 0;
    u8 xTile3 = 0;
    u8 yTile = 0;
    u8 yTile2 = 0;
    u8 oneYTile = FALSE;
    u8 oneXTile = FALSE;
    u16 shipxw = playerShip.object.position.x + playerShip.object.width;
    u8 shipyh = playerShip.object.position.y + playerShip.object.height;

    //get the enemy count for checking enemy colisions
    u8 enemyCount = listItemCount(&enemyList);
    u8 i = 0;

    //update previous position
    playerShip.previousPosition.x = playerShip.object.position.x;
    playerShip.previousPosition.y = playerShip.object.position.y;
            
    //calculate new x position from x velocity, going right
    if(playerShip.object.velocity.x > 0)
    {
        xTile = (shipxw + playerShip.object.velocity.x) / TILE_WIDTH_BYTES;        
        yTile = (playerShip.object.position.y) / TILE_HEIGHT;
        yTile2 = (shipyh) / TILE_HEIGHT;

        //tileScrollX*2 is added to include scrolling for the maximum edge of the screen
        if(playerShip.object.position.x < SCREEN_X + tileScrollX*4 - playerShip.object.width)
        {                                    
            if(MAPA(xTile,yTile) == 255 && MAPA(xTile,yTile2) == 255)
            {
                playerShip.object.position.x += playerShip.object.velocity.x;
                positionChanged_ = TRUE;
            }
            else
            {
                hitSolid_ = TRUE;
            }
        }             
    }
    else if(playerShip.object.velocity.x < 0) //going left
    {
        //handle the fact we are using a u16 which means that 1 - 2 = 65536
        //not the most elegant solution and an i16 might be a fine solution
        //but this does solve the problem
        if(playerShip.object.position.x + playerShip.object.velocity.x < UNSIGNED_NEGATIVE)
        {
            xTile = (playerShip.object.position.x + playerShip.object.velocity.x) / TILE_WIDTH_BYTES;       
            yTile = playerShip.object.position.y / TILE_HEIGHT;
            yTile2 = shipyh / TILE_HEIGHT;

            //tileScrollX*2 is included to set as the maximum left when scrolling is active
            if(playerShip.object.position.x > 0 + tileScrollX*4 )
            {           
                if(MAPA(xTile,yTile) == 255 && MAPA(xTile,yTile2) == 255)
                {
                    playerShip.object.position.x += playerShip.object.velocity.x;
                    positionChanged_ = TRUE;
                }
            } 
        }         
    }

    //calculate new y position from y velocity (going down)
    if(playerShip.object.velocity.y > 0)
    {               
        yTile = (shipyh + playerShip.object.velocity.y) / TILE_HEIGHT;
        xTile = (playerShip.object.position.x) / TILE_WIDTH_BYTES; 
        xTile2 = (shipxw) / TILE_WIDTH_BYTES;
        
        if(xTile2 - xTile > 1)
        {
            xTile3 = xTile2 - 1;
        }
        else
        {
            xTile3 = xTile2;
        }
        
        if(playerShip.object.position.y < SCREEN_Y - playerShip.object.height)
        {
            if(MAPA(xTile,yTile) == 255 && MAPA(xTile2,yTile) == 255 && MAPA(xTile3,yTile) == 255)
            {
                playerShip.object.position.y += playerShip.object.velocity.y;
                positionChanged_ = TRUE;
            }
        }        
    }
    else if(playerShip.object.velocity.y < 0) //going up
    {
        yTile = (playerShip.object.position.y + playerShip.object.velocity.y) / TILE_HEIGHT;
        xTile = (playerShip.object.position.x) / TILE_WIDTH_BYTES; 
        xTile2 = (shipxw) / TILE_WIDTH_BYTES;
        
        if(xTile2 - xTile > 1)
        {
            xTile3 = xTile2 - 1;
        }
        else
        {
            xTile3 = xTile2;
        }

        if(playerShip.object.position.y > 0)
        {
            if(MAPA(xTile,yTile) == 255 && MAPA(xTile2,yTile) == 255 && MAPA(xTile3,yTile) == 255)
            {
                playerShip.object.position.y += playerShip.object.velocity.y;
                positionChanged_ = TRUE;
            }
        }        
    }


    //look for enemies to crash into
    for(i = 0; i<enemyCount; ++i)
    {
        Enemy* enemy = listItemAt(&enemyList, i);
        
        if(shipxw >= enemy->object.position.x &&
            playerShip.object.position.x <= enemy->object.position.x + enemy->object.width)
        {
            if(shipyh >= enemy->object.position.y &&
                playerShip.object.position.y <= enemy->object.position.y + enemy->object.height)
            {
                //crashed into enemy

                //make sure the enemy hasnt previously been destroyed
                if(enemy->destroyed == FALSE)
                {
                    //lose a life for crashing into an enemy
                    --playerShip.lifeCount;

                    //force redraw of the hud
                    gameHud.redraw = TRUE;

                    //set the enemy to destroyed
                    enemy->destroyed = TRUE;
                }                    
            }
        }
    }   

    //instant death on running into the boss
    if(mapHasBoss)
    {
        //make sure the boss hasnt been destroyed alreay
        if(bossAlive)
        {
            if(shipxw >= bossPosition.x &&
                playerShip.object.position.x <= bossPosition.x + BOSS_WIDTH)
            {
                if(shipyh >= bossPosition.y &&
                    playerShip.object.position.y <= bossPosition.y + BOSS_HEIGHT)
                {
                    playerShip.lifeCount = 0;
                }
            }
        }
        
    }

    if(positionChanged_)
    {               
        u8 powerupCount = listItemCount(&powerupList);

        //look for powerups
        for(i = 0; i<powerupCount; ++i)
        {
            PowerUp* powerup = listItemAt(&powerupList, i);

            if(shipxw >= powerup->object.position.x &&
               playerShip.object.position.x <= powerup->object.position.x + powerup->object.width)
            {
                if(shipyh >= powerup->object.position.y &&
                   playerShip.object.position.y <= powerup->object.position.y + powerup->object.height)
                {                    
                    //interact with powerup
                    switch(powerup->type)
                    {
                        case SPEED:
                        {
                            //update the score once you pickup the item
                            addToScore(&gameHud, powerup->score);

                            powerup->used = TRUE;

                            //increase the ship speed
                            playerShip.shipSpeed = 2;
                        }
                        break;

                        case POWER:
                        {
                            //update the score once you pickup the item
                            addToScore(&gameHud, powerup->score);

                            //5 seems like a good maximum
                            if(playerShip.power < MAXIMUM_SHOOT_POWER_)
                            {
                                ++playerShip.power;
                            }
                            
                            powerup->used = TRUE;
                        }
                        break;

                        case EXTRALIFE:
                        {
                            //update the score once you pickup the item
                            addToScore(&gameHud, powerup->score);

                            //9 lives is the maximum because we only want to show one digit on the hud
                            if(playerShip.lifeCount < MAXIMUM_LIFE_COUNT_)
                            {
                                ++playerShip.lifeCount;
                            }

                            powerup->used = TRUE;
                        }
                        break;
                    }               
                }
            }
        }   

    //only start scrolling once the ship has reached the centre of the screen (including scrolling)
            //the >= -2 corrects for when moving at speed
        if(playerShip.object.position.x + playerShip.object.width - xScrollOffset > HALF_SCREEN_X)
        {

            //need to stop scrolling once the scrollx reaches the maximum
            //the max being total - first screen width
            if(tileScrollX < mapXTileCount -  SCREEN_TILES_X)
            {
                i8 deltax = (playerShip.object.position.x - playerShip.previousPosition.x);

                //make sure we scroll the screen at the same speed as the ship
                //xScrollOffset += deltax;//playerShip.shipSpeed;

                //we have to apply the scrolloffset for every value if we skip one value ie +=2 it will break scrolling because of the %4
                for(i=0; i<deltax; ++i)
                {
                    ++xScrollOffset;

                    if((xScrollOffset % TILE_WIDTH_BYTES) == 0 && !finishedScrolling)
                    {    
                        //move the screen to provide hardware scrolling 
                        if(xScrollOffset)
                        {
                            //xscrolloffset does actually go over which is a bit of a problem but not for the momenmt as we cant even get past a single screen yet
                            u8 newtest = xScrollOffset / 2;
                            ++tileScrollX;
                            mapRedraw = TRUE;
                            
                            //redraw the boss on scroll
                            if(mapHasBoss)
                            {
                                bossRedraw = TRUE;
                            }

                            //not entirely sure why i need to divide this by 2 but it works
                            cpct_setVideoMemoryOffset(newtest);//xScrollOffset/2);
                        }                    
                    }
                }
                


            }
            else
            {
                //we dont want to reset scrolling once we get to the edge as this asumes that
                //the map is a fixed length where this would actually works
                //xScrollOffset = 0;
                finishedScrolling = TRUE;
            }
    /*
            if(xScrollOffset > 80)
            {
                xScrollOffset -= 80;
            }*/


            
        }    

        //ship reaches the end of the level
        if(levelEnd != -1)
        {
            if(xTile >= levelEnd || xTile2 >= levelEnd)
            {
                playerShip.reachedLevelEnd = TRUE;
            }
        }
    }

    //velocity set to 0 after position changed
    playerShip.object.velocity.x = 0;
    playerShip.object.velocity.y = 0;    
}

void moveShip(u8 direction)
{
    if(direction == LEFT)
    {
        playerShip.object.velocity.x = -playerShip.shipSpeed;
        playerShip.direction = LEFT;
    }
    else if(direction == RIGHT)
    {
        playerShip.object.velocity.x = playerShip.shipSpeed;
        playerShip.direction = RIGHT;
    }
    else if(direction == UP)
    {
        playerShip.object.velocity.y = -playerShip.shipSpeed;
    }
    else if(direction == DOWN)
    {
        playerShip.object.velocity.y = playerShip.shipSpeed;
    }    
}

//shoot a projectile from the centre of the ship
void _shootProjectile(Ship* ship)
{        
    Projectile p;

    p.object.position.x = ship->object.position.x + ship->object.width/2;
    p.object.position.y = ship->object.position.y + ship->object.height/2;
    p.direction = ship->direction;
    p.parent = SHIPPROJECTILE;

    initProjectile(&p);

    //add it to the list
    if(projectileTimerReady)
    {
        if(listItemCount(&projectileList) < 5)
        {
            addListItemCopy(&projectileList, &p, sizeof(Projectile));
        } 
        projectileTimerReady = FALSE;
    }
}